@@ -2136,6 +2136,10 @@ class NullSafeNEQ(Binary, Predicate):
     pass
 
 
+class Distance(Binary):
+    pass
+
+
 class Escape(Binary):
     pass
 
@@ -2160,18 +2164,14 @@ class Is(Binary, Predicate):
     pass
 
 
+class Kwarg(Binary):
+    """Kwarg in special functions like func(kwarg => y)."""
+
+
 class Like(Binary, Predicate):
     pass
 
 
-class SimilarTo(Binary, Predicate):
-    pass
-
-
-class Distance(Binary):
-    pass
-
-
 class LT(Binary, Predicate):
     pass
 
@@ -2192,6 +2192,10 @@ class NEQ(Binary, Predicate):
     pass
 
 
+class SimilarTo(Binary, Predicate):
+    pass
+
+
 class Sub(Binary):
     pass
 
